@@ -5,47 +5,51 @@ use strict;
 use warnings;
 
 sub description { 'Microsoft Exchange Server 2007' }
-sub make {
+sub inquire {
     # Detect an error from Microsoft Exchange Server 2007
     # @param    [Hash] mhead    Message headers of a bounce email
     # @param    [String] mbody  Message body of a bounce email
     # @return   [Hash]          Bounce data list and message/rfc822 part
-    # @return   [Undef]         failed to parse or the arguments are missing
+    # @return   [undef]         failed to parse or the arguments are missing
     # @since v4.1.1
     my $class = shift;
     my $mhead = shift // return undef;
     my $mbody = shift // return undef;
+    my $match = 0;
 
     # Content-Language: en-US, fr-FR
-    return undef unless $mhead->{'subject'} =~ /\A(?:Undeliverable|Non_remis_|Non[ ]recapitabile):/;
+    $match ||= 1 if index($mhead->{'subject'}, 'Undeliverable')    == 0;
+    $match ||= 1 if index($mhead->{'subject'}, 'Non_remis_')       == 0;
+    $match ||= 1 if index($mhead->{'subject'}, 'Non recapitabile') == 0;
+    return undef unless $match > 0;
+
     return undef unless defined $mhead->{'content-language'};
-    return undef unless $mhead->{'content-language'} =~ /\A[a-z]{2}(?:[-][A-Z]{2})?\z/;
+    $match += 1 if length $mhead->{'content-language'} == 2; # JP
+    $match += 1 if length $mhead->{'content-language'} == 5; # ja-JP
+    return undef unless $match > 1;
 
     # These headers exist only a bounce mail from Office365
     return undef if $mhead->{'x-ms-exchange-crosstenant-originalarrivaltime'};
     return undef if $mhead->{'x-ms-exchange-crosstenant-fromentityheader'};
 
     state $indicators = __PACKAGE__->INDICATORS;
-    state $rebackbone = qr{^(?:
-         Original[ ]message[ ]headers:                  # en-US
-        |En-t.tes[ ]de[ ]message[ ]d'origine[ ]:        # fr-FR/En-têtes de message d'origine
-        |Intestazioni[ ]originali[ ]del[ ]messaggio:    # it-CH
-        )
-    }mx;
+    state $boundaries = [
+        'Original message headers:',                # en-US
+        "tes de message d'origine :",               # fr-FR/En-têtes de message d'origine
+        'Intestazioni originali del messaggio:',    # it-CH
+    ];
     state $markingsof = {
-        'message' => qr{\A(?:
-             Diagnostic[ ]information[ ]for[ ]administrators:               # en-US
-            |Informations[ ]de[ ]diagnostic[ ]pour[ ]les[ ]administrateurs  # fr-FR
-            |Informazioni[ ]di[ ]diagnostica[ ]per[ ]gli[ ]amministratori   # it-CH
-            )
-        }x,
-        'error'   => qr/[ ]((?:RESOLVER|QUEUE)[.][A-Za-z]+(?:[.]\w+)?);/,
-        'rhost'   => qr{\A(?:
-             Generating[ ]server            # en-US
-            |Serveur[ ]de[ ]g.+ration[ ]    # fr-FR/Serveur de génération
-            |Server[ ]di[ ]generazione      # it-CH
-            ):[ ]?(.*)
-        }x,
+        'message' => [
+            'Diagnostic information for administrators:',           # en-US
+            'Informations de diagnostic pour les administrateurs',  # fr-FR
+            'Informazioni di diagnostica per gli amministratori',   # it-CH
+        ],
+        'error'   => [' RESOLVER.', ' QUEUE.'],
+        'rhost'   => [
+            'Generating server',        # en-US
+            'Serveur de g',             # fr-FR/Serveur de g辿n辿ration
+            'Server di generazione',    # it-CH
+        ],
     };
     state $ndrsubject = {
         'SMTPSEND.DNS.NonExistentDomain'=> 'hostunknown',   # 554 5.4.4 SMTPSEND.DNS.NonExistentDomain
@@ -63,7 +67,7 @@ sub make {
     };
 
     my $dscontents = [__PACKAGE__->DELIVERYSTATUS];
-    my $emailsteak = Sisimai::RFC5322->fillet($mbody, $rebackbone);
+    my $emailparts = Sisimai::RFC5322->part($mbody, $boundaries);
     my $readcursor = 0;     # (Integer) Points the current cursor position
     my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
     my $connvalues = 0;     # (Integer) Flag, 1 if all the value of $connheader have been set
@@ -72,12 +76,12 @@ sub make {
     };
     my $v = undef;
 
-    for my $e ( split("\n", $emailsteak->[0]) ) {
-        # Read error messages and delivery status lines from the head of the email
-        # to the previous line of the beginning of the original message.
+    for my $e ( split("\n", $emailparts->[0]) ) {
+        # Read error messages and delivery status lines from the head of the email to the previous
+        # line of the beginning of the original message.
         unless( $readcursor ) {
             # Beginning of the bounce message or message/delivery-status part
-            $readcursor |= $indicators->{'deliverystatus'} if $e =~ $markingsof->{'message'};
+            $readcursor |= $indicators->{'deliverystatus'} if grep { index($e, $_) == 0 } $markingsof->{'message'}->@*;
             next;
         }
         next unless $readcursor & $indicators->{'deliverystatus'};
@@ -93,57 +97,68 @@ sub make {
             # Original message headers:
             $v = $dscontents->[-1];
 
-            if( $e =~ /\A([^ @]+[@][^ @]+)\z/ ) {
+            if( index($e, ' ') < 0 && index($e, '@') > 1 ) {
                 # kijitora@example.jp
                 if( $v->{'recipient'} ) {
                     # There are multiple recipient addresses in the message body.
                     push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
                     $v = $dscontents->[-1];
                 }
-                $v->{'recipient'} = $1;
+                $v->{'recipient'} = Sisimai::Address->s3s4($e);
                 $recipients++;
 
-            } elsif( $e =~ /([45]\d{2})[ ]([45][.]\d[.]\d+)[ ].+\z/ ) {
-                # #550 5.1.1 RESOLVER.ADR.RecipNotFound; not found ##
-                # #550 5.2.3 RESOLVER.RST.RecipSizeLimit; message too large for this recipient ##
-                # Remote Server returned '550 5.1.1 RESOLVER.ADR.RecipNotFound; not found'
-                # 3/09/2016 8:05:56 PM - Remote Server at mydomain.com (10.1.1.3) returned '550 4.4.7 QUEUE.Expired; message expired'
-                $v->{'replycode'} = int $1;
-                $v->{'status'}    = $2;
-                $v->{'diagnosis'} = $e;
-
             } else {
-                # Continued line of error messages
-                next unless $v->{'diagnosis'};
-                next unless substr($v->{'diagnosis'}, -1, 1) eq '=';
-                substr($v->{'diagnosis'}, -1, 1, $e);
+                my $cr = Sisimai::SMTP::Reply->find($e)  || '';
+                my $cs = Sisimai::SMTP::Status->find($e) || '';
+                if( $cr || $cs ) {
+                    # #550 5.1.1 RESOLVER.ADR.RecipNotFound; not found ##
+                    # #550 5.2.3 RESOLVER.RST.RecipSizeLimit; message too large for this recipient ##
+                    # Remote Server returned '550 5.1.1 RESOLVER.ADR.RecipNotFound; not found'
+                    # 3/09/2016 8:05:56 PM - Remote Server at mydomain.com (10.1.1.3) returned '550 4.4.7 QUEUE.Expired; message expired'
+                    $v->{'replycode'} = $cr;
+                    $v->{'status'}    = $cs;
+                    $v->{'diagnosis'} = $e;
+
+                } else {
+                    # Continued line of error messages
+                    next unless $v->{'diagnosis'};
+                    next unless substr($v->{'diagnosis'}, -1, 1) eq '=';
+                    substr($v->{'diagnosis'}, -1, 1, $e);
+                }
             }
         } else {
             # Diagnostic information for administrators:
             #
             # Generating server: mta22.neko.example.org
-            next unless $e =~ $markingsof->{'rhost'};
+            next unless grep { index($e, $_) == 0 } $markingsof->{'rhost'}->@*;
             next if $connheader->{'rhost'};
-            $connheader->{'rhost'} = $1;
+            $connheader->{'rhost'} = substr($e, index($e, ':') + 1,);
             $connvalues++;
         }
     }
     return undef unless $recipients;
 
     for my $e ( @$dscontents ) {
-        if( $e->{'diagnosis'} =~ $markingsof->{'error'} ) {
-            # #550 5.1.1 RESOLVER.ADR.RecipNotFound; not found ##
-            my $f = $1;
-            for my $r ( keys %$ndrsubject ) {
-                # Try to match with error subject strings
-                next unless $f eq $r;
-                $e->{'reason'} = $ndrsubject->{ $r };
-                last;
-            }
-        }
+        my $p = -1;
+
         $e->{'diagnosis'} = Sisimai::String->sweep($e->{'diagnosis'});
+        for my $q ( $markingsof->{'error'}->@* ) {
+        # Find an error message, get an error code
+            $p = index($e->{'diagnosis'}, $q);
+            last if $p > -1;
+        }
+        next unless $p > 0;
+
+        # #550 5.1.1 RESOLVER.ADR.RecipNotFound; not found ##
+        my $f = substr($e->{'diagnosis'}, $p + 1, index($e->{'diagnosis'}, ';') - $p - 1);
+        for my $r ( keys %$ndrsubject ) {
+            # Try to match with error subject strings
+            next unless $f eq $r;
+            $e->{'reason'} = $ndrsubject->{ $r };
+            last;
+        }
     }
-    return { 'ds' => $dscontents, 'rfc822' => $emailsteak->[1] };
+    return { 'ds' => $dscontents, 'rfc822' => $emailparts->[1] };
 }
 
 1;
@@ -153,8 +168,7 @@ __END__
 
 =head1 NAME
 
-Sisimai::Lhost::Exchange2007 - bounce mail parser class for C<Microsft Exchange
-Server 2007>.
+Sisimai::Lhost::Exchange2007 - bounce mail parser class for C<Microsft Exchange Server 2007>.
 
 =head1 SYNOPSIS
 
@@ -162,8 +176,7 @@ Server 2007>.
 
 =head1 DESCRIPTION
 
-Sisimai::Lhost::Exchange2007 parses a bounce email which created by C<Microsoft
-Exchange Server 2007>.
+Sisimai::Lhost::Exchange2007 parses a bounce email which created by C<Microsoft Exchange Server 2007>.
 Methods in the module are called from only Sisimai::Message.
 
 =head1 CLASS METHODS
@@ -174,10 +187,10 @@ C<description()> returns description string of this module.
 
     print Sisimai::Lhost::Exchange2007->description;
 
-=head2 C<B<make(I<header data>, I<reference to body string>)>>
+=head2 C<B<inquire(I<header data>, I<reference to body string>)>>
 
-C<make()> method parses a bounced email and return results as a array reference.
-See Sisimai::Message for more details.
+C<inquire()> method parses a bounced email and return results as a array reference. See Sisimai::Message
+for more details.
 
 =head1 AUTHOR
 
@@ -185,7 +198,7 @@ azumakuniyuki
 
 =head1 COPYRIGHT
 
-Copyright (C) 2016-2020 azumakuniyuki, All rights reserved.
+Copyright (C) 2016-2021,2023 azumakuniyuki, All rights reserved.
 
 =head1 LICENSE
 
